/*
 * Author: 黎永顺
 * name: 薪资核算-列表
 * Description:
 * Date: 2023/9/14
 */
import React, { FunctionComponent, useContext, useEffect, useMemo, useRef, useState } from "react";
import { Button, Dropdown, Form, Input, InputNumber, MenuProps, Space, Table, Tooltip, Typography } from "antd";
import { DeleteOutlined, LockOutlined, QuestionCircleOutlined, SettingOutlined, UnlockOutlined } from "@ant-design/icons";
import type { FormInstance } from "antd/es/form";
import CustomTableTitle from "@/pages/calcTable/customTableTitle";
import CalcExplainFooter from "@/pages/calcTable/calcExplainFooter";
import CaclFixedTotal from "./calcFixedTotal";
import type { ColumnType } from "antd/lib/table";
import type { PaginationData } from "rc-pagination";
import { exceptStr, paginationFun, toDecimal_n } from "@/utils/common";
import { IPage } from "@/common/types";
import styles from "@/pages/atdTable/components/index.less";

interface OwnProps {}

interface EditableRowProps {}

interface EditableCellProps {
  editCell?: any;
  children?: React.ReactNode;
  record?: any;
  pattern?: number;
  rowIndex?: number;
  dataIndex?: any;
  handleSave?: (type: string, record: any) => void;
}

type Props = OwnProps;
type fixedProps = boolean | "top" | "bottom";
const { Text } = Typography;

const EditableContext = React.createContext<FormInstance<any> | null>(null);
const Index: FunctionComponent<Props> = (props) => {
  const [columns, setColumns] = useState<ColumnType<any>[]>([]);
  const [dataSource, setDataSource] = useState<any[]>([]);
  const [selectedRowKeys, setSelectedRowKeys] = useState<React.Key[]>([]);
  const [pageInfo, setPageInfo] = useState<Partial<PaginationData>>({});
  const [i18n, setI18n] = useState<any>({});
  const [showTotalCell, setShowTotalCell] = useState<boolean>(false);
  const [isDetailTable, setIsDetailTable] = useState<boolean>(false);
  const [showSee, setShowSee] = useState<boolean>(false);
  const [sumRowlistUrl, setSumRowlistUrl] = useState<string>("");
  const [tableScrollHeight, setTableScrollHeight] = useState<Number>(0);
  const [payload, setPayload] = useState<string>("");
  const [fixed, setFixed] = useState<fixedProps>(true);
  const [optWidth, setOptWidth] = useState<number>(120); //操作列宽度
  const [sumRow, setSumRow] = useState<Partial<{}>>({}); //总计行数据
  const [editCell, setEditCell] = useState<any>({}); //编辑行key
  const [form] = Form.useForm();

  useEffect(() => {
    window.parent.postMessage({ type: "init" }, "*");
    window.addEventListener("message", receiveMessageFromIndex, false);
    return () => {
      window.removeEventListener("message", receiveMessageFromIndex, false);
    };
  }, []);
  const receiveMessageFromIndex = (event: any) => {
    const data: any = exceptStr(event.data);
    if (!_.isEmpty(data)) {
      const {
        columns,
        dataSource,
        pageInfo,
        selectedRowKeys: rowKeys = [],
        i18n: i18nRes = {},
        showTotalCell = false,
        sumRowlistUrl = "",
        payload = {},
        calcDetail,
        showSee = false,
        fixed = true,
        tableScrollHeight,
        sumRow,
        optWidth
      } = data;
      setSumRowlistUrl(sumRowlistUrl);
      setShowTotalCell(showTotalCell);
      setIsDetailTable(calcDetail);
      setShowSee(showSee);
      setI18n(i18nRes);
      setPayload(payload);
      setFixed(fixed);
      setSumRow(sumRow);
      setPageInfo(pageInfo);
      setDataSource(dataSource);
      setOptWidth(optWidth);
      setSelectedRowKeys([...selectedRowKeys, ...rowKeys]);
      setTableScrollHeight(tableScrollHeight);
      setColumns(_.map(columns, (o) => ({ ...o, i18n: i18nRes })));
      setEditCell({});
    }
  };
  const convertColumns: any = (cols: any[]) => {
    return _.map(cols, (item) => {
      if (_.isNaN(parseInt(item.dataIndex))) {
        return { ...item };
      } else {
        return {
          ...item,
          title: <CustomTableTitle {...item} onHandleFormulatd={handleFormulaTd} />,
          children: convertColumns(_.map(item.children, (o) => ({ ...o, i18n: item.i18n }))),
          className: styles["td_odd"],
          i18n: item.i18n,
          onCell: (record: any, rowIndex: number) => ({
            className: !_.isEmpty(record[`${item.dataIndex}_feedback`]) && styles["feedbackBg"],
            editCell,
            record,
            rowIndex,
            pattern: item.pattern,
            dataIndex: item.dataIndex,
            onContextMenu: (e: any) => {
              (!item.calcDetail || item.rightClickType) && e.preventDefault();
            },
            onDoubleClick: (e: any) => {
              e.preventDefault();
              !item.calcDetail || item.rightClickType ? setEditCell({ dataIndex: item?.dataIndex, rowIndex }) : null;
            },
            handleSave
          }),
          render: (text: string, record: any) => {
            let items: MenuProps["items"] = !item.calcDetail
              ? [
                  {
                    label: item.i18n["锁定"],
                    key: "LOCK",
                    icon: <LockOutlined />,
                    onClick: () =>
                      handleLockEmp({
                        lockStatus: "LOCK",
                        acctEmpId: record?.id,
                        salaryItemId: item?.dataIndex
                      })
                  },
                  {
                    label: item.i18n["解锁"],
                    key: "UNLOCK",
                    icon: <UnlockOutlined />,
                    onClick: () =>
                      handleLockEmp({
                        lockStatus: "UNLOCK",
                        acctEmpId: record?.id,
                        salaryItemId: item?.dataIndex
                      })
                  }
                ]
              : [];
            item.rightClickType &&
              item.rightClickType.includes("DELFEEDBACK") &&
              (items = [
                ...items,
                {
                  label: item.i18n["删除反馈"],
                  key: "DELFEEDBACK",
                  icon: <DeleteOutlined />,
                  onClick: () =>
                    handleDelFeedback({
                      acctEmpId: record?.id,
                      salaryItemId: item?.dataIndex
                    })
                }
              ]);
            item.rightClickType &&
              item.rightClickType.includes("SETFEEDBACK") &&
              (items = [
                ...items,
                {
                  label: item.i18n["设置反馈"],
                  key: "SETFEEDBACK",
                  icon: <SettingOutlined />,
                  onClick: () =>
                    handleSetFeedback({
                      acctEmpId: record?.id,
                      salaryItemId: item?.dataIndex
                    })
                }
              ]);
            return (
              <Dropdown menu={{ items: !item.calcDetail || item.rightClickType ? items : [] }} trigger={["contextMenu"]} overlayClassName={styles.contextMenu} destroyPopupOnHide>
                <span className={styles.contentSpan}>
                  <span title={text} className={styles.contentTitle} style={{ color: `${record?.[item.dataIndex + "_color"]}` }}>
                    {text}
                  </span>
                  {record.lockItems && record.lockItems.includes(item.dataIndex) ? <LockOutlined title={item.i18n["锁定的项目值"]} /> : null}
                  {!_.isEmpty(record[`${item.dataIndex}_feedback`]) ? (
                    <Tooltip title={record[`${item.dataIndex}_feedback`]}>
                      <QuestionCircleOutlined />
                    </Tooltip>
                  ) : null}
                </span>
              </Dropdown>
            );
          }
        };
      }
    });
  };
  const handleSave = (salaryItemId: string, row: any) => {
    const params = { salaryItemId, idList: [row?.id], value: row[salaryItemId] };
    window.parent.postMessage({ type: "turn", payload: { id: "EDITCELL", params } }, "*");
  };
  const handleFormulaTd = (dataIndex: string) => {
    window.parent.postMessage({ type: "turn", payload: { id: "FORMULA", params: { dataIndex } } }, "*");
  };
  const handleEdit = (id: string, showSee: boolean) => {
    window.parent.postMessage({ type: "turn", payload: { id: "EDIT", params: { id, showSee } } }, "*");
  };
  const handleLockEmp = (params: any) => {
    window.parent.postMessage({ type: "turn", payload: { id: "LOCKEMP", params } }, "*");
  };
  // 复旦大学附属医院二开-薪资核算项添加右击删除反馈的操作
  const handleDelFeedback = (params: any) => {
    window.parent.postMessage({ type: "turn", payload: { id: "DELFEEDBACK", params } }, "*");
  };
  const handleSetFeedback = (params: any) => {
    window.parent.postMessage({ type: "turn", payload: { id: "SETFEEDBACK", params } }, "*");
  };
  const sizeChange = (pageobj: IPage) => {};
  const onChange = (pageobj: IPage) => {
    setPageInfo(() => {
      window.parent.postMessage({ type: "turn", payload: { id: "PAGEINFO", params: { ...pageInfo, ...pageobj } } }, "*");
      return { ...pageInfo, ...pageobj };
    });
  };
  const rowSelection = {
    columnWidth: 50,
    columnTitle: isDetailTable ? "序号" : "",
    renderCell: (value: boolean, record: any, index: number, originNode: React.ReactNode) => (isDetailTable ? <span>{index + 1}</span> : originNode),
    selectedRowKeys,
    preserveSelectedRowKeys: true,
    onChange: (rowKeys: React.Key[]) => {
      setSelectedRowKeys(rowKeys);
      window.parent.postMessage(
        {
          type: "turn",
          payload: { id: "CHECKBOX", params: { selectedRowKeys: rowKeys } }
        },
        "*"
      );
    }
  };
  const cols = useMemo(() => {
    return [
      ...convertColumns(columns),
      {
        title: i18n["操作"],
        dataIndex: "operate",
        fixed: "right",
        width: optWidth || 120,
        render: (__: any, record: any) => (
          <Space>
            <Button type="link" onClick={() => handleEdit(record?.id, showSee)}>
              {showSee ? i18n["查看"] : i18n["编辑"]}
            </Button>
            {optWidth && (
              <>
                <Button
                  type="link"
                  onClick={() =>
                    handleLockEmp({
                      lockStatus: "LOCK",
                      acctEmpIds: [record?.id]
                    })
                  }
                >
                  {i18n["锁定"]}
                </Button>
                <Button
                  type="link"
                  onClick={() =>
                    handleLockEmp({
                      lockStatus: "UNLOCK",
                      acctEmpIds: [record?.id]
                    })
                  }
                >
                  {i18n["解锁"]}
                </Button>
              </>
            )}
            {record?.lockTime && <Text>{record?.lockTime}</Text>}
          </Space>
        )
      }
    ];
  }, [columns, editCell, i18n]);
  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell
    }
  };
  return (
    <Form form={form} component={false}>
      <Table
        rowKey="id"
        size="small"
        bordered
        className={styles.tableWrapper}
        dataSource={dataSource}
        rowSelection={rowSelection}
        components={components}
        scroll={{
          x: 1200,
          y: `calc(100vh - ${tableScrollHeight || (!showTotalCell ? 165 : 200)}px)`
        }}
        columns={!isDetailTable || showSee ? cols : _.filter(cols, (o) => o.dataIndex !== "operate")}
        footer={() => (!isDetailTable ? <CalcExplainFooter i18n={i18n} /> : null)}
        pagination={
          !_.isNil(pageInfo)
            ? {
                ...paginationFun(pageInfo, sizeChange, onChange, i18n),
                size: "default"
              }
            : false
        }
        summary={() =>
          !showTotalCell ? (
            <></>
          ) : (
            <Table.Summary fixed={fixed}>
              <Table.Summary.Row>
                <Table.Summary.Cell index={0} align="center">
                  <Text type="danger">{i18n["总计"]}</Text>
                </Table.Summary.Cell>
                <CaclFixedTotal columns={cols} dataSourceUrl={sumRowlistUrl} payload={payload} sumRow={sumRow} />
              </Table.Summary.Row>
            </Table.Summary>
          )
        }
      />
    </Form>
  );
};

export default Index;
const EditableRow: React.FC<EditableRowProps> = (props) => {
  const [form] = Form.useForm();
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  );
};
const EditableCell: React.FC<EditableCellProps> = (props) => {
  const { children, editCell, record, dataIndex, rowIndex, handleSave, ...restProps } = props;
  const form = useContext(EditableContext)!;
  const inputRef = useRef<any>(null);
  useEffect(() => {
    if (dataIndex && editCell && editCell.dataIndex === dataIndex && editCell.rowIndex === rowIndex) {
      toggleEdit();
      inputRef.current!.focus();
    }
  }, [editCell, dataIndex, rowIndex]);
  const toggleEdit = () => {
    form.setFieldsValue({ [dataIndex]: record[dataIndex] });
  };
  const onSave = _.debounce(async (e) => {
    try {
      let values = await form.validateFields();
      if (record[`${_.keys(values)[0]}_type`] === "number") {
        values = {
          ...values,
          [_.keys(values)[0]]: !!_.get(values, _.keys(values)[0]) || _.get(values, _.keys(values)[0]) === 0 ? toDecimal_n(_.get(values, _.keys(values)[0]), restProps?.pattern) : ""
        };
      }
      handleSave?.(_.keys(values)[0], { ...record, ...values });
    } catch (errInfo) {
      console.log("Save failed:", errInfo);
    }
  }, 200);
  let childNode =
    dataIndex && editCell && editCell.dataIndex === dataIndex && editCell.rowIndex === rowIndex ? (
      <Form.Item style={{ margin: 0 }} name={dataIndex}>
        {record[`${dataIndex}_type`] === "number" ? (
          <InputNumber ref={inputRef} onPressEnter={onSave} onBlur={onSave} precision={restProps?.pattern} />
        ) : (
          <Input value={record[dataIndex]} onPressEnter={onSave} onBlur={onSave} ref={inputRef} />
        )}
      </Form.Item>
    ) : (
      <>{children}</>
    );
  return <td {...restProps}>{childNode}</td>;
};
